/*
Copyright (C) 2000  Andreas Richter

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <stdio.h>
#include <math.h>

#define maxpointszeta 700
#define maxpointseta 150
#define wedge 80
#define delta 15
#define theta 45
#define wedgex 0.25
#define wedgey 0
#define PI 3.1415926	/* might become handy */
/* Wegde */
#define a1 0.24931
#define b1 0.220321
#define c1 (-212.213)
#define d1 92.9215
#define a2 0.769572
#define b2 0.343608
#define c2 (-226.467)
#define d2 114.263
/* ramp */
    /* bottom */
#define arb1 0.323964
#define brb1 0.14335
#define crb1 85.2367
#define drb1 72.9822
#define arb2 0.677141
#define brb2 0.143649
#define crb2 -15.4713
#define drb2 73.4721
    /* top */
#define art 0.813063
#define brt 0.356012
#define crt 211.933
#define drt 182.14
    /* left -right */
#define arl 0.490513
#define brl 0.745383 
#define crl 49.2062
#define drl 62.3614
/* number of gridpoints, for which these constants were found */
/* user entered number has to be in relationship to these numbers */
#define zetadef 300
#define etadef 100

double XY[maxpointszeta][maxpointseta][2];
double c[maxpointszeta][maxpointseta];
double U[maxpointszeta][maxpointseta];
double dxdzeta[maxpointszeta][maxpointseta][3];
double dydeta[maxpointszeta][maxpointseta][3];
double dxdeta[maxpointszeta][maxpointseta][3];
double dydzeta[maxpointszeta][maxpointseta][3];

int zetamax, etamax;
double xmax, ymax;

double zetaratio, etaratio;

double x(double zeta, double a, double b, double c, double d)
{
  return(a+b*tan((zeta-c)/d));
}

double y(double eta, double a, double b, double c, double d)
{
  return(a+b*atan((eta-c)/d));
}


void bottom()
{
  int zeta,eta;
  double xcoord, ycoord;
  double pos;
  /* boundaries */
  XY[0][0][0] = 0;
  XY[0][0][1] = 0;
  XY[zetamax-1][0][0] = 1;
  XY[zetamax-1][0][1] = cos(delta*PI/180)*(1-wedgex);
  eta = 0;
  for (zeta=1; zeta<=zetamax;zeta++)
  {
    if (zeta<=wedge)
    {
      XY[zeta-1][eta][0]=x((zeta),a1,b1,c1,d1)*(double)xmax;
      XY[zeta-1][eta][1]=0;
    }
    else
    {
      pos = x((zeta),a1,b1,c1,d1);
      xcoord = (wedgex+(pos-wedgex)*cos(PI/180*delta))*(double)xmax;
      ycoord = sin(PI/180*delta)*(pos-wedgex)*(double)ymax;
      XY[zeta-1][eta][0]=xcoord;
      XY[zeta-1][eta][1]=ycoord;
    }
  }         
}

void top()
{  
  int zeta,eta;
  /* boundaries */
  XY[0][etamax-1][0] = 0;
  XY[0][etamax-1][1] = (double)ymax;
  XY[zetamax-1][etamax-1][0] = (double)xmax;
  XY[zetamax-1][etamax-1][1] = (double)ymax;
  eta = etamax-1;
  for (zeta=1; zeta<=zetamax; zeta++)
  {
    XY[zeta-1][eta][0]=x((zeta),a2,b2,c2,d2)*(double)xmax;
    XY[zeta-1][eta][1]=(double)ymax;
  }
}

void connecttopbottom()
{
  int zeta, eta;
  double xlength, ylength;
  for (zeta=0; zeta<zetamax; zeta++)
  {
    xlength = (XY[zeta][0][0]-XY[zeta][etamax-1][0]);
    ylength = (XY[zeta][0][1]-XY[zeta][etamax-1][1]);
   /* printf("%f %f, ", XY[zeta][0][0],XY[zeta][0][1]); */
    printf("%f %f\n", XY[zeta][etamax-1][0],XY[zeta][etamax-1][1]);
    for (eta=1; eta<etamax-1; eta++)
    { 
      XY[zeta][eta][0]=XY[zeta][0][0]-xlength/(etamax-1)*eta;
      XY[zeta][eta][1]=XY[zeta][0][1]-ylength/(etamax-1)*eta;
    }
  }
}

void bottomramp()
{
  int zeta, eta;
  double xcoord, ycoord;
  eta=0;
  for (zeta=1; zeta<=(int)(89*zetaratio); zeta++)
  {
    XY[zeta][eta][1]=0;
    XY[zeta][eta][0]=x((zeta+1)/zetaratio,arb1,brb1,crb1,drb1)*(double)xmax;      
  }
  for (zeta=(int)(89*zetaratio); zeta<=(int)(150*zetaratio); zeta++)
  {
    xcoord=x((zeta+1)/zetaratio,arb1,brb1,crb1,drb1)*(double)xmax;
    XY[zeta][eta][0]=xcoord;
    ycoord=tan(15*PI/180)*(xcoord-XY[(int)(89*zetaratio)][0][0]);
    XY[zeta][eta][1]=ycoord; 
  }
  for (zeta=(int)(150*zetaratio); zeta<=(int)(209*zetaratio); zeta++)
  {
    xcoord=x((zeta+1)/zetaratio,arb2,brb2,crb2,drb2)*(double)xmax;
    XY[zeta][eta][0]=xcoord;
    ycoord=tan(15*PI/180)*(xcoord-XY[(int)(89*zetaratio)][0][0]);
    XY[zeta][eta][1]=ycoord;
  }
  for (zeta=(int)(209*zetaratio); zeta<=zetamax-1; zeta++)
  {
    XY[zeta][eta][0]=x((zeta+1)/zetaratio,arb2,brb2,crb2,drb2)*(double)xmax;
    XY[zeta][eta][1]=ycoord;
  }
  XY[0][0][0]=0;
  XY[0][0][1]=0;
  XY[zetamax-1][0][0]=(double)xmax;
  XY[zetamax-1][0][1]=ycoord;
}

void bottomramp2()
{
  int zeta,eta;
  for (zeta=1;zeta<=(zetamax)*2/7+1;zeta++)
  {
    XY[zeta][0][0]=xmax/3/(double)((zetamax)*2/7)*((double)zeta);
    XY[zeta][0][1]=0;
  }
  for (zeta=zetamax*2/7+2;zeta<(zetamax*5/7-2);zeta++)
  {
    XY[zeta][0][0]=xmax/3+xmax/3/((zetamax)*3/7)
                   *((double)zeta-(double)((zetamax)*2/7));
    XY[zeta][0][1]=tan(15*PI/180)*
                   (XY[zeta][0][0]-XY[zetamax*2/7+1][0][0]);    
  }
  for(zeta=rint(5*(zetamax)/7)-2;zeta<zetamax;zeta++)
  {
    XY[zeta][0][0]=2*xmax/3+xmax/3/(double)((zetamax+2)*2/7)*
                   ((double)zeta-(double)((zetamax)*5/7-1));
    XY[zeta][0][1]=tan(15*PI/180)*
                   (XY[zetamax*5/7-2][0][0]-XY[zetamax*2/7+1][0][0]);
  }
}

void topramp()
{
  int zeta, eta;  
  char ch;  
  while (ch!='\n')
    ch = getchar();
  printf("Even or shockwave adapted distribution at the top? (E/S)");
  ch=getchar();
  if ((ch=='s')||(ch=='S'))
  {
    printf("Shockwave adapted\n");
    eta=etamax-1;
    for(zeta=1; zeta<=zetamax-1;zeta++)
    {
      XY[zeta][eta][1]=(double)ymax;
      XY[zeta][eta][0]=x((zeta+1)/zetaratio,art,brt,crt,drt)*(double)xmax;
    }
    XY[0][etamax-1][0]=0;
    XY[0][etamax-1][1]=(double)ymax;
    XY[zetamax-1][etamax-1][0]=(double)xmax;
    XY[zetamax-1][etamax-1][1]=(double)ymax;   
  }
  else 
  {
    for (zeta=0;zeta<=zetamax-1;zeta++)
    {
      XY[zeta][etamax-1][1]=ymax;
      XY[zeta][etamax-1][0]=xmax/(zetamax-1)*(double)zeta;
    }
  }
  while (ch!='\n')
    ch = getchar();
}

void verticalramp()
{
  int zeta,eta;
  double dx;
  zeta=0;
  /* left boundary */
  for(eta=1; eta<=etamax-2;eta++)
  {
    XY[zeta][eta][0]=0;
    XY[zeta][eta][1]=(double)ymax*y((eta+1)/etaratio,arl,brl,crl,drl);
  }
  /* start of wedge*/
  zeta=(int)(89*zetaratio);
  dx=(XY[zeta][etamax-1][0]-XY[zeta][0][0]);
  for(eta=1; eta<=etamax-2;eta++)
  {
    XY[zeta][eta][1]=y((eta+1)/etaratio,arl,brl,crl,drl)
        *(double)ymax;
    XY[zeta][eta][0]=XY[zeta][0][0]
      +dx*XY[zeta][eta][1]/((double)ymax);
  }
  /* end of wedge */
  zeta=(int)(209*zetaratio);
  dx=(XY[zeta][etamax-1][0]-XY[zeta][0][0])/
      (XY[zeta][etamax-1][1]-XY[zeta][0][1]);
  for(eta=1; eta<=etamax-2;eta++)
  {
    XY[zeta][eta][1]=((double)ymax-XY[zeta][0][1])
                     *y((eta+1)/etaratio,arl,brl,crl,drl)+XY[zeta][0][1];
    XY[zeta][eta][0]=XY[zeta][0][0]+
                     dx*(XY[zeta][eta][1]-XY[zeta][0][1]);
  }
  /* right boundary */
  zeta=zetamax-1;
  for(eta=1; eta<=etamax-2;eta++)
  {
    XY[zeta][eta][0]=(double)xmax;
    XY[zeta][eta][1]=((double)ymax-XY[zeta][0][1])
                     *y((eta+1)/etaratio,arl,brl,crl,drl)+XY[zeta][0][1];
  } 
}

void connect()
{
  double m1,m2,n1,n2;
  int zeta,eta,i;
  double zetaleft, zetaright;
  for (i=1; i<=3;i++)
  {
    switch (i)
    {
      case 1 : zetaleft = 0; zetaright = 89*zetaratio; break;
      case 2 : zetaleft = 89*zetaratio; zetaright = 209*zetaratio; break;
      case 3 : zetaleft = 209*zetaratio; zetaright = zetamax-1; break;
    }
    for(zeta=(int)(zetaleft)+1;zeta<(int)zetaright; zeta++)
    {
      m1 = (XY[zeta][etamax-1][1]-XY[zeta][0][1])/
            (XY[zeta][etamax-1][0]-XY[zeta][0][0]);
      n1 = XY[zeta][etamax-1][1]-m1*XY[zeta][etamax-1][0];
      for(eta=1;eta<=etamax-2;eta++)
      {
        m2 = (XY[(int)zetaright][eta][1]-XY[(int)zetaleft][eta][1])/
             (XY[(int)zetaright][eta][0]-XY[(int)zetaleft][eta][0]);
        n2 = XY[(int)zetaleft][eta][1]-XY[(int)zetaleft][eta][0]*m2;
        XY[zeta][eta][0]=(n2-n1)/(m1-m2);
        XY[zeta][eta][1]=XY[zeta][eta][0]*m1+n1;
      }
    }
  }
}

void rectangular()
{
  int zeta,eta;
  double dx, dy;
//  dx= (float)xmax/(zetamax-1);
//  dy= (float)ymax/(etamax-1);
	dx = (float)1/(float)(zetamax-1);
	dy = (float)1/(float)(etamax-0.5);
  for (zeta=0;zeta<zetamax;zeta++)
    for (eta=0; eta<etamax;eta++)
    {
      XY[zeta][eta][0]=dx*zeta;
//      XY[zeta][eta][1]=dy*(double)(eta+0.5);
      XY[zeta][eta][1]=(-1)*(dy*eta)*(dy*eta)+2*(dy*eta);
      printf("%e\n", XY[zeta][eta][1]);
    }
}

void concentration()
{
	int xi, eta;
	for (xi=0;xi<zetamax;xi++)
		for (eta=0; eta<etamax; eta++)
			c[xi][eta]=1;

}

void flowrate()
{
  char ch;
  int xi, xistart=0, xiend, eta, countsection=1;
  int OK=1;
  double start=0, end, step, vel,xiendreal;
  step = (double)1/(double)(zetamax-1);
  printf("%f\n",step);
  while (ch!='\n')
    ch=getchar();
  printf("Use different flow rates at different portions of the pipe?");
  ch = getchar();
  if ((ch=='Y')||(ch=='y')) {
    while (ch!='\n')
      ch=getchar();
    do {
    	OK=1;
		  printf("%d. section: Starting at x = %f, ending at x = ?",
		  	countsection, start);
		  scanf("%lf",&end);
		  if (end<=start) OK=0;
		  else {
		    if (end>=1){
		    	end=1.0;
		    }
		    else OK=0;
  		  xiendreal = (end/step);
  		  xiend = (int)(ceil(xiendreal));
  		  printf("%d\n",xiend);
  		  printf("Enter a value from 0 to 1 for the velocity:");
  		  scanf("%lf",&vel);
  		  for (xi=xistart;xi<=xiend;xi++)
	  	  	for (eta=0;eta<etamax;eta++)
		    		U[xi][eta]=vel;
	  	  xistart=xiend;
	  	  start=end;
	  	  countsection++;
	  	}
		} while (OK==0);
  }
  else {
    while (ch!='\n')
      ch=getchar();
   for (xi=0;xi<zetamax;xi++)
   		for (eta=0;eta<etamax;eta++)
   		  U[xi][eta]=1.0;
   printf("%lf\n",U[1][1]);
	}
}

void tmAnalytically()
{
  int zeta,eta;
  double sec1,sec2,K;
  double tan1,tan2;
  for(zeta=0; zeta<=zetamax-1;zeta++)
  {
    for(eta=0; eta<=etamax-1;eta++)
    {  
       sec1=1/(cos((zeta-c1)/d1)*cos((zeta-c1)/d1));
       sec2=1/(cos((zeta-c2)/d2)*cos((zeta-c2)/d2));
       tan1=b1*tan((zeta-c1)/d1);
       tan2=b2*tan((zeta-c2)/d2);
       /* derivative of zeta wrt x */
       /* derivative of eta wrt to x */
       /* derivative of eta wrt y */
       /* and derivative of zeta wrt to y*/
       if (zeta<= wedge)
       {
         K=(b1*sec1/d1)+eta/etamax*(-b1*sec1/d1+b2*sec2/d2);
         dxdzeta[zeta][eta][0]=K;
         K=(-a1+a2-tan1+tan2)/etamax;
         dxdeta[zeta][eta][0]= K;
         K=(double)(1.00000/etamax);
         dydeta[zeta][eta][0]=K;
         dydzeta[zeta][eta][0]=0;
       }
       else
       {
         K=(b1*sec1*cos(delta*PI/180))/d1+eta/etamax*(-b1*cos(delta*PI/180)*sec1/d1+b2*sec2/d2);
         dxdzeta[zeta][eta][0]=K;
         K=(-0.25+a2-cos(delta*PI/180)*(-0.25+a1+tan1)+tan2)/etamax;
         dxdeta[zeta][eta][0]=K;
         K=(1-tan(delta*PI/180)*cos(delta*PI/180)*(-wedgex+a1+tan1))/etamax;
         dydeta[zeta][eta][0]=K; 
         K=(b1*sec1*sin(delta*PI/180))/d1*(1-eta/etamax);
         dydzeta[zeta][eta][0]=K;
       }
    }
  }
}

void tmFirstOrder()
{
/* for the numerical differentiation the following can be stated: */
/*      deta/dy = dzeta/dy     */
/* 	deta/dx = dzeta/dx     */
/* because deta = dzeta =1 for all points on the grid. */
  int zeta,eta;
  double x1, x2, y1, y2;
  for(eta=0;eta<=etamax-1;eta++)
  {
    /* backward difference for last column for dxdzeta and dydzeta*/
    dxdzeta[zetamax-1][eta][1]=XY[zetamax-1][eta][0]-XY[zetamax-2][eta][0]; 
    dydzeta[zetamax-1][eta][1]=XY[zetamax-1][eta][1]-XY[zetamax-2][eta][1];
    /* rest: forward difference */
    if (eta<etamax-1)
    {
      dydeta[zetamax-1][eta][1]=XY[zetamax-1][eta+1][1]-XY[zetamax-1][eta][1];
      dxdeta[zetamax-1][eta][1]=XY[zetamax-1][eta+1][0]-XY[zetamax-1][eta][0];
    }    
  }
  for(zeta=0; zeta<=zetamax-1;zeta++)
  { 
    /* backward difference for last row for dydeta and dxdeta */
    dydeta[zeta][etamax-1][1]=XY[zeta][etamax-1][1]-XY[zeta][etamax-2][1]; 
    dxdeta[zeta][etamax-1][1]=XY[zeta][etamax-1][0]-XY[zeta][etamax-2][0];
    /* rest: forward difference */
    if (zeta<zetamax-1)
    {
      dxdzeta[zeta][etamax-1][1]=XY[zeta+1][etamax-1][0]-XY[zeta][etamax-1][0]; 
      dydzeta[zeta][etamax-1][1]=XY[zeta+1][etamax-1][1]-XY[zeta][etamax-1][1];
    }
  }
  for(zeta=0; zeta<zetamax-1;zeta++)
  {
    for(eta=0;eta<etamax-1;eta++)
    {
      /* forward difference for the rest of the elements */
      dxdzeta[zeta][eta][1]=XY[zeta+1][eta][0]-XY[zeta][eta][0]; 
      dydzeta[zeta][eta][1]=XY[zeta+1][eta][1]-XY[zeta][eta][1];
      dydeta[zeta][eta][1]=XY[zeta][eta+1][1]-XY[zeta][eta][1]; 
      dxdeta[zeta][eta][1]=XY[zeta][eta+1][0]-XY[zeta][eta][0];
    }
  }
}

void tmSecondOrder()
{
  int zeta,eta;
  zeta=0;
  for(eta=0;eta<=etamax-1;eta++)
  {
    /* backward difference for last column for dxdzeta and dydzeta*/
    dxdzeta[zetamax-1][eta][2]=(3*XY[zetamax-1][eta][0]-4*XY[zetamax-2][eta][0]+XY[zetamax-3][eta][0])/2;
    dydzeta[zetamax-1][eta][2]=(3*XY[zetamax-1][eta][1]-4*XY[zetamax-2][eta][1]+XY[zetamax-3][eta][1])/2;
    /* forward difference for first column  for dxdzeta and dydzeta*/
    dxdzeta[0][eta][2]=(-XY[2][eta][0]+4*XY[1][eta][0]-3*XY[0][eta][0])/2; 
    dydzeta[0][eta][2]=(-XY[2][eta][1]+4*XY[1][eta][1]-3*XY[0][eta][1])/2;
    /* rest: central difference: */
    if ((eta<etamax-1)&&(eta>0))
    {
      dydeta[zetamax-1][eta][2]=(XY[zetamax-1][eta+1][1]-XY[zetamax-1][eta-1][1])/2;
      dydeta[0][eta][2]=(XY[0][eta+1][1]-XY[0][eta-1][1])/2;
      dxdeta[zetamax-1][eta][2]=(XY[zetamax-1][eta+1][0]-XY[zetamax-1][eta-1][0])/2;
      dxdeta[0][eta][2]=(XY[0][eta+1][0]-XY[0][eta-1][0])/2;
    }  
  }
  for(zeta=0; zeta<=zetamax-1;zeta++)
  {
    /* backward difference for last row for dydeta and dxdeta*/
    dydeta[zeta][etamax-1][2]=(3*XY[zeta][etamax-1][1]-4*XY[zeta][etamax-2][1]+XY[zeta][etamax-3][1])/2;
    dxdeta[zeta][etamax-1][2]=(3*XY[zeta][etamax-1][0]-4*XY[zeta][etamax-2][0]+XY[zeta][etamax-3][0])/2;
    /* forward difference for first row for dydeta and dxdeta*/ 
    dydeta[zeta][0][2]=(-XY[zeta][2][1]+4*XY[zeta][1][1]-3*XY[zeta][0][1])/2; 
    dxdeta[zeta][0][2]=(-XY[zeta][2][0]+4*XY[zeta][1][0]-3*XY[zeta][0][0])/2;
    /* rest: central difference */
    if ((zeta<zetamax-1)&&(zeta>0))
    {
      dxdzeta[zeta][etamax-1][2]=(XY[zeta+1][etamax-1][0]-XY[zeta-1][etamax-1][0])/2;
      dydzeta[zeta][etamax-1][2]=(XY[zeta+1][etamax-1][1]-XY[zeta-1][etamax-1][1])/2;
      dxdzeta[zeta][0][2]=(XY[zeta+1][0][0]-XY[zeta-1][0][0])/2;
      dydzeta[zeta][0][2]=(XY[zeta+1][0][1]-XY[zeta-1][0][1])/2;
    }
  }
  for(zeta=1; zeta<zetamax-1;zeta++)
  {
    for(eta=1;eta<etamax-1;eta++)
    {
      /* central difference for the rest of the elements */
      dxdzeta[zeta][eta][2]=(XY[zeta+1][eta][0]-XY[zeta-1][eta][0])/2;
      dydzeta[zeta][eta][2]=(XY[zeta+1][eta][1]-XY[zeta-1][eta][1])/2;
      dydeta[zeta][eta][2]=(XY[zeta][eta+1][1]-XY[zeta][eta-1][1])/2;
      dxdeta[zeta][eta][2]=(XY[zeta][eta+1][0]-XY[zeta][eta-1][0])/2;
      
      /* forward difference for the rest of the elements */
      /*dxdzeta[0][eta][2]=2/(-XY[2][eta][0]+4*XY[1][eta][0]-3*XY[0][eta][0]); */
    }
  }
}

void output_tecplot()
{
  int i,j;
  FILE *outputfile;  
  outputfile = fopen("problem3.plt","w");
  fprintf(outputfile, "TITLE=\"Data File\"\n");
  fprintf(outputfile, "VARIABLES=\"X\" \"Y\" ");
  fprintf(outputfile, "\"dxdzeta_a\" \"dydeta_a\" ");
  fprintf(outputfile, "\"dydzeta_a\" \"dxdeta_a\" ");
  fprintf(outputfile, "\"dxdzeta_1\" \"dydeta_1\" ");
  fprintf(outputfile, "\"dydzeta_1\" \"dxdeta_1\" ");
  fprintf(outputfile, "\"dxdzeta_2\" \"dydeta_2\" ");
  fprintf(outputfile, "\"dydzeta_2\" \"dxdeta_2\"\n"); 
  fprintf(outputfile, "ZONE I=%d J=%d F=POINT\n",etamax, zetamax);
  for (i=0; i<zetamax; i++)
  {
    for (j=0; j<etamax; j++)
    {
      /* X,Y Coordinates */	
      fprintf(outputfile,"%lf %lf ",XY[i][j][0],XY[i][j][1]);
      /* Analytical solution */
      fprintf(outputfile,"%lf %lf ",dxdzeta[i][j][0], dydeta[i][j][0]);
      fprintf(outputfile,"%lf %lf ",dydzeta[i][j][0], dxdeta[i][j][0]);
      /* first order finite difference */
      fprintf(outputfile,"%lf %lf ",dxdzeta[i][j][1], dydeta[i][j][1]);
      fprintf(outputfile,"%lf %lf ",dydzeta[i][j][1], dxdeta[i][j][1]);
      /* second order finite difference */
      fprintf(outputfile,"%lf %lf ",dxdzeta[i][j][2], dydeta[i][j][2]);
      fprintf(outputfile,"%lf %lf\n",dydzeta[i][j][2], dxdeta[i][j][2]);
    }
  }
  fclose(outputfile);
}

void outputdata()
{
  FILE *outf;
  int i,j;
  double a,b;
  char filename[255];
  printf("Filename to be used for output data file: ");
  scanf("%s",&filename);
  outf = fopen(filename,"w");
  fprintf(outf,"Xmax = %d\n",zetamax);
  fprintf(outf,"Rmax = %d\n",etamax);
  fprintf(outf,"x  y  c  X  U\n");
  for (i=0; i<zetamax; i++)
  {
    for (j=0; j<etamax; j++)
    {
      fprintf(outf,"%lf %lf ", XY[i][j][0],XY[i][j][1]);
      fprintf(outf,"%lf %lf %lf\n", c[i][j], 0.0, U[i][j]);
/*      fprintf(outf,"%lf %lf ",dxdzeta[i][j][2],dydzeta[i][j][2]);
      fprintf(outf,"%lf %lf\n",dxdeta[i][j][2],dydeta[i][j][2]); */
    }
  }
  fclose(outf);
}


void main()
{
  char c;
  printf("Number of gridpoints in computational space (zeta*eta):");
  scanf("%d*%d",&zetamax,&etamax);
  zetaratio = (double)zetamax/zetadef;
  etaratio = (double)etamax/etadef;
//  printf("Create Rectangular, Wedge or rAmp/channel configuration (R/W/A) ?");
//  c = getchar();
  printf("Please enter the coordinates (in physical space) of the upper right corner\n");
  printf("as xmax*ymax:");
  scanf("%lf*%lf",&xmax, &ymax);
//  if ((c=='R')||(c=='r'))
//  {
    rectangular();
    concentration();
    flowrate();
//  }
//  else if ((c=='w')||(c=='w'))
//  {
//    bottom();
//    top();
//    connecttopbottom();
//  }
//  else
//  {
//    bottomramp2();
//    printf("bottom .... OK\n");
//    topramp();
//    printf("top .... OK\n");
//    verticalramp();
//    printf("vertical .... OK\n");
//    connect();
//  }
//  tmAnalytically();
//  tmFirstOrder();
//  tmSecondOrder();
  outputdata();
  // output_tecplot(); 
}

